#
#  Copyright (c) 2020 Appfire Technologies, Inc.
#  All rights reserved.
#  This software is licensed under the provisions of the "Bob Swift Atlassian Add-ons EULA"
#  (https://bobswift.atlassian.net/wiki/x/WoDXBQ) as well as under the provisions of
#  the "Standard EULA" from the "Atlassian Marketplace Terms of Use" as a "Marketplace Product”
#  (http://www.atlassian.com/licensing/marketplace/termsofuse).
#  See the LICENSE file for more details.
#

import click
import subprocess
import os
import sys
import questionary
import logging
import yaml
import argparse


@click.command()
@click.option('--verbose', '-v', is_flag=True, help="Verbose output")
@click.option('--account', '-a', help="AWS account number")
@click.option('--region', '-r', help="AWS region", default="us-east-1")
@click.option('--profile', '-p', help="AWS profile as the default environment", default="default")
@click.option('--env', '-e', help="personal, dev, test, stage or prod", default="personal")
@click.option('--stack', '-s', help="CDK stack to deploy", default="*")
@click.argument('command')
def process(verbose, command, account, region, profile, env, stack):
    """
    gathers information related to deployment and deploys the CDK stack
    """

    logging.basicConfig(level=logging.INFO, stream=sys.stdout, format="%(message)s")
    logger = logging.getLogger("ac_app_deploy")
    if env == 'personal':
        personal_env_settings = None
        with open("./personal.env.yml", 'r') as stream:
            try:
                personal_env_settings = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print(exc)
        aws_profile = profile if profile else personal_env_settings['environment']['personal']['profile']
        aws_app_domain = personal_env_settings['environment']['personal']['AWS_APP_DOMAIN']
        logger.info(f'Environment: {env}')
        logger.info(f'AWS Account: {account}')
        logger.info(f'AWS Profile: {aws_profile}')
        logger.info(f'Domain: {aws_app_domain}')
        logger.info(f'AWS Region: {region}')
    else:
        envSettings = None
        # variables
        with open("./env.yml", 'r') as stream:
            try:
                envSettings = yaml.safe_load(stream)
            except yaml.YAMLError as exc:
                print(exc)
        aws_profile = profile if profile else envSettings['environment'][env]['profile']
        aws_app_domain = envSettings['environment'][env]['domain']
        logger.info(f'Environment: {env}')
        logger.info(f'AWS Account: {account}')
        logger.info(f'AWS Profile: {aws_profile}')
        logger.info(f'Domain: {aws_app_domain}')
        logger.info(f'AWS Region: {region}')

    os.environ['DOMAIN'] = aws_app_domain
    os.environ['STAGE'] = env

    if command == 'bootstrap':
        if aws_app_domain is None and os.environ.get('DOMAIN') is None:
            aws_app_domain = questionary.text("The Domain name of your Application?").ask()
        if region is None and os.environ.get('CDK_DEPLOY_REGION') is None:
            region = questionary.text("The default region you want to deploy to?").ask()
        if account is None and os.environ.get('CDK_DEPLOY_ACCOUNT') is None:
            account = questionary.text("The AWS account you want to deploy to?").ask()
        os.environ['CDK_DEPLOY_REGION'] = region
        os.environ['CDK_DEPLOY_ACCOUNT'] = account
        os.environ['DOMAIN'] = aws_app_domain
        os.environ['STAGE'] = env
        shell(logger, "cdk bootstrap && cdk deploy AcBizServices AppServices* ModuleServices*")
    if command == 'deploy':
        if aws_app_domain is None and os.environ.get('DOMAIN') is None:
            aws_app_domain = questionary.text("The Domain name of your Application?").ask()
        if region is None and os.environ.get('CDK_DEPLOY_REGION') is None:
            region = questionary.text("The default region you want to deploy to?").ask()
        if account is None and os.environ.get('CDK_DEPLOY_ACCOUNT') is None:
            account = questionary.text("The AWS account you want to deploy to?").ask()
        os.environ['CDK_DEPLOY_REGION'] = region
        os.environ['CDK_DEPLOY_ACCOUNT'] = account
        shell(logger, "cdk deploy " + stack, raise_error=True)
    elif command == 'diff':
        shell(logger, "cdk diff", raise_error=True)
    elif command == 'synth':
        shell(logger, "cdk synth", raise_error=True)
    elif command == 'destroy':
        shell(logger, "cdk destroy " + stack, raise_error=True)
    else:
        shell(logger, "cdk list", raise_error=True)


def shell(logger, cmd, raise_error=False):
    """
    Run a shell command.
    :param logger:
    :param cmd:  Shell line to be executed
    :param raise_error:
    :return: Tuple (return code, interleaved stdout and stderr output as string)
    """

    logger.debug("Running : %s" % cmd)

    # process = subprocess.Popen(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)

    process = subprocess.run(
        cmd,
        check=True,
        shell=True
    )

    # :E1103: *%s %r has no %r member (but some types could not be inferred)*
    # pylint: disable=E1103
    # out = out.decode("utf-8")
    # err = err.decode("utf-8")
    # print(out)
    if raise_error and process.returncode != 0:
        logger.error("Command output:")
        # logger.error(out + err)
        raise ShellCommandFailed("The following command did not succeed: %s" % cmd)

    return (process.returncode)


class ShellCommandFailed(Exception):
    """ Executing a shell command failed """


if __name__ == "__main__":
    process()
